@@ -3,7 +3,7 @@
 #' 
 #' @param HMMfits List comprised of \code{\link{momentuHMM}} objects
 #' @param alpha Significance level for calculating confidence intervals of pooled estimates (including location error ellipses). Default: 0.95.
-#' @param ncores Number of cores to use for parallel processing.
+#' @param ncores Number of cores to use for parallel processing. Default: 1 (no parallel processing).
 #' @param covs Data frame consisting of a single row indicating the covariate values to be used in the calculation of pooled natural parameters. 
 #' For any covariates that are not specified using \code{covs}, the means of the covariate(s) across the imputations are used 
 #' (unless the covariate is a factor, in which case the first factor in the data is used). By default, no covariates are specified.
@@ -33,12 +33,12 @@
 #' err.model <- miExample$err.model
 #' 
 #' # Fit crawl to obsData
-#' crwOut <- crawlWrap(obsData,ncores=1,theta=c(4,0),fixPar=c(1,1,NA,NA),
+#' crwOut <- crawlWrap(obsData,theta=c(4,0),fixPar=c(1,1,NA,NA),
 #'                     initial.state=inits,err.model=err.model)
 #'                     
 #' # Fit four imputations
 #' bPar <- miExample$bPar
-#' HMMfits <- MIfitHMM(crwOut,nSims=4,ncores=1,poolEstimates=FALSE,
+#' HMMfits <- MIfitHMM(crwOut,nSims=4,poolEstimates=FALSE,
 #'                    nbStates=2,dist=list(step="gamma",angle="vm"),
 #'                    Par0=bPar$Par,beta0=bPar$beta,delta0=bPar$delta,
 #'                    formula=~cov1+cos(cov2),
@@ -46,7 +46,7 @@
 #'                    covNames=c("cov1","cov2"))
 #'                    
 #' # Pool estimates
-#' miSum <- MIpool(HMMfits,ncores=1)
+#' miSum <- MIpool(HMMfits)
 #' print(miSum)
 #' }
 #' @export
@@ -58,9 +58,10 @@
 #' @importFrom car dataEllipse
 #' @importFrom mitools MIcombine
 #' @importFrom MASS ginv
-MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
+MIpool<-function(HMMfits,alpha=0.95,ncores=1,covs=NULL){
   
   im <- HMMfits
+  goodIndex <- 1:length(im)
   simind <- which((unlist(lapply(im,is.momentuHMM))))
   nsims <- length(simind)
   if(nsims<1) stop("'HMMfits' must be a list comprised of momentuHMM objects")
@@ -69,6 +70,7 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
   if(length(checkmove)) {
     im[checkmove]<-NULL
     warning("The following imputations are not momentuHMM objects and will be ignored: ",paste(checkmove,collapse=", "))
+    goodIndex <- goodIndex[-checkmove]
   }
   checksims <- lapply(im,function(x) x[match("conditions",names(x))])
   ident <- !unlist(lapply(checksims,function(x) isTRUE(all.equal(x,checksims[[1]]))))
@@ -79,21 +81,55 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
     if(any(ident2)) stop("Model conditions for each imputation must be identical. Imputations that do not match the first: ",paste(which(ident),collapse=", "))
   }
   
+  if(any(unlist(lapply(im,function(x) is.null(x$mod$hessian))))) stop("Estimates cannot be pooled unless Hessian is calculated. Hessian is missing for imputations ",paste0(which(unlist(lapply(im,function(x) is.null(x$mod$hessian)))),collapse=", "))
+  
   tmpDet <- which(unlist(lapply(im,function(x) det(x$mod$hessian)))==0)
   if(length(tmpDet)){
-    warning("Hessian is singular for HMM fit(s): ",paste0(tmpDet,collapse=", "))
+    warning("Hessian is singular for HMM fit(s): ",paste0(goodIndex[tmpDet],collapse=", "))
   }
   
   tmpVar <- which(unlist(lapply(im,function(x) any(class(tryCatch(ginv(x$mod$hessian),error=function(e) e)) %in% "error"))))
   if(length(tmpVar)){
-    warning("ginv of the hessian failed for HMM fit(s): ",paste0(tmpVar,collapse=", "))
+    warning("ginv of the hessian failed for HMM fit(s): ",paste0(goodIndex[tmpVar],collapse=", "))
     im[tmpVar] <- NULL
     nsims <- length(im)
+    goodIndex <- goodIndex[-tmpVar]
   }
   
   im <- lapply(im,delta_bc)
-  
   m <- im[[1]]
+  
+  tempcons<-rep(1,length(m$mod$estimate))
+  tempworkcons<-rep(0,length(m$mod$estimate))
+  tempcons[1:length(unlist(m$conditions$cons))]<-unlist(m$conditions$cons)
+  tempworkcons[1:length(unlist(m$conditions$workcons))]<-unlist(m$conditions$workcons)
+  
+  wBounds <- cbind(unlist(lapply(m$conditions$workBounds,function(x) x[,1])),unlist(lapply(m$conditions$workBounds,function(x) x[,2])))
+  
+  # check for finite coefficients and standard errors
+  betaVar <- lapply(im,function(x) get_gradwb(x$mod$estimate,wBounds,tempcons)%*%ginv(x$mod$hessian)%*%t(get_gradwb(x$mod$estimate,wBounds,tempcons)))
+  betaCoeff <- lapply(im,function(x) w2wn(x$mod$estimate^tempcons+tempworkcons,wBounds))
+  tmpVar1 <- which(unlist(lapply(betaCoeff,function(x) any(!is.finite(x)))))
+  if(length(tmpVar1)){
+    warning("working parameter estimates are not finite for HMM fits ",paste0(goodIndex[tmpVar1],collapse=", ")," and will not be included in pooling")
+    im[tmpVar1] <- NULL
+    m <- im[[1]]
+    nsims <- length(im)
+    betaVar <- lapply(im,function(x) get_gradwb(x$mod$estimate,wBounds,tempcons)%*%ginv(x$mod$hessian)%*%t(get_gradwb(x$mod$estimate,wBounds,tempcons)))
+    betaCoeff <- lapply(im,function(x) w2wn(x$mod$estimate^tempcons+tempworkcons,wBounds))
+    goodIndex <- goodIndex[-tmpVar1]
+  }
+  tmpVar2 <- which(unlist(lapply(betaVar,function(x) any(!is.finite(x)))))
+  if(length(tmpVar2)){
+    warning("working parameter standard errors are not finite for HMM fits ",paste0(goodIndex[tmpVar2],collapse=", ")," and will not be included in pooling")
+    im[tmpVar2] <- NULL
+    m <- im[[1]]
+    nsims <- length(im)
+    betaCoeff <- lapply(im,function(x) w2wn(x$mod$estimate^tempcons+tempworkcons,wBounds))
+    betaVar <- lapply(im,function(x) get_gradwb(x$mod$estimate,wBounds,tempcons)%*%ginv(x$mod$hessian)%*%t(get_gradwb(x$mod$estimate,wBounds,tempcons)))
+    goodIndex <- goodIndex[-tmpVar2]
+  }
+  
   data <- m$data
   nbStates <- length(m$stateNames)
   dist <- m$conditions$dist
@@ -122,23 +158,23 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
   parms <- names(m$CIbeta)
   nparms <- length(parms)
   xmat <- xbar <- xvar <- W_m <- B_m <- MI_se <- lower <- upper <- list()
-  parmcols <- lapply(m$conditions$fullDM,ncol)#
+  parCount <- lapply(m$conditions$fullDM,ncol)#
+  for(i in distnames[unlist(m$conditions$circularAngleMean)]){
+    parCount[[i]] <- length(unique(gsub("cos","",gsub("sin","",colnames(m$conditions$fullDM[[i]])))))
+  }
+  parmcols <- parCount
   parmcols$beta <- ncol(m$mle$beta)
   parmcols$delta <- nbStates-1
   parmcols <- unlist(parmcols[parms])
   
-  parindex <- c(0,cumsum(c(unlist(lapply(m$conditions$fullDM,ncol)),length(m$mle$beta),ncol(m$covsDelta)*(nbStates-1))))
+  parindex <- c(0,cumsum(c(unlist(parCount),length(m$mle$beta),ncol(m$covsDelta)*(nbStates-1))))
   names(parindex)[1:length(distnames)] <- distnames
   if(nbStates>1) {
     names(parindex)[length(distnames)+1] <- "beta"
     names(parindex)[length(parindex)-1] <- "delta"
   }
   
-  tempcons<-rep(1,length(m$mod$estimate))
-  tempworkcons<-rep(0,length(m$mod$estimate))
-  tempcons[1:length(unlist(m$conditions$cons))]<-unlist(m$conditions$cons)
-  tempworkcons[1:length(unlist(m$conditions$workcons))]<-unlist(m$conditions$workcons)
-  miBeta <- mitools::MIcombine(results=lapply(im,function(x) x$mod$estimate^tempcons+tempworkcons),variances=lapply(im,function(x) diag(tempcons*(x$mod$estimate^(tempcons-1)))%*%ginv(x$mod$hessian)%*%t(diag(tempcons*(x$mod$estimate^(tempcons-1))))))
+  miBeta <- mitools::MIcombine(results=betaCoeff,variances=betaVar)
   
   for(parm in 1:nparms){
     
@@ -221,23 +257,25 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
   }
   
   tmPar <- lapply(m$mle[distnames],function(x) c(t(x)))
-  parindex <- c(0,cumsum(unlist(lapply(m$conditions$fullDM,ncol)))[-length(m$conditions$fullDM)])
+  parindex <- c(0,cumsum(unlist(parCount))[-length(m$conditions$fullDM)])
   names(parindex) <- distnames
   for(i in distnames){
     if(!is.null(m$conditions$DM[[i]])){# & m$conditions$DMind[[i]]){
-      tmPar[[i]] <- m$mod$estimate[parindex[[i]]+1:ncol(m$conditions$fullDM[[i]])]
-      names(tmPar[[i]])<-colnames(m$conditions$fullDM[[i]])
-    } else if((m$conditions$dist[[i]] %in% angledists) & (!m$conditions$estAngleMean[[i]])){
+      tmPar[[i]] <- m$mod$estimate[parindex[[i]]+1:parCount[[i]]]
+      if(m$conditions$circularAngleMean[[i]]){
+        names(tmPar[[i]]) <- unique(gsub("cos","",gsub("sin","",colnames(m$conditions$fullDM[[i]]))))
+      } else names(tmPar[[i]])<-colnames(m$conditions$fullDM[[i]])
+    } else if((dist[[i]] %in% angledists) & (!m$conditions$estAngleMean[[i]])){
       tmPar[[i]] <- tmPar[[i]][-(1:nbStates)]
     }
   }
   
-  inputs <- checkInputs(nbStates,m$conditions$dist,tmPar,m$conditions$estAngleMean,m$conditions$circularAngleMean,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$DM,m$conditions$userBounds,NULL,NULL,m$stateNames)
+  inputs <- checkInputs(nbStates,dist,tmPar,m$conditions$estAngleMean,m$conditions$circularAngleMean,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$DM,m$conditions$userBounds,NULL,NULL,m$stateNames)
   p<-inputs$p
   splineInputs<-getSplineDM(distnames,inputs$DM,m,tempCovs)
-  DMinputs<-getDM(splineInputs$covs,splineInputs$DM,m$conditions$dist,nbStates,p$parNames,p$bounds,tmPar,NULL,NULL,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$circularAngleMean)
+  DMinputs<-getDM(splineInputs$covs,splineInputs$DM,inputs$dist,nbStates,p$parNames,p$bounds,tmPar,NULL,NULL,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$circularAngleMean)
   fullDM <- DMinputs$fullDM
-  #DMinputs<-getDM(tempCovs,inputs$DM,m$conditions$dist,nbStates,p$parNames,p$bounds,tmPar,m$conditions$cons,m$conditions$workcons,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$circularAngleMean)
+  #DMinputs<-getDM(tempCovs,inputs$DM,inputs$dist,nbStates,p$parNames,p$bounds,tmPar,m$conditions$cons,m$conditions$workcons,m$conditions$zeroInflation,m$conditions$oneInflation,m$conditions$circularAngleMean)
   #fullDM<-DMinputs$fullDM
   
   formula<-m$conditions$formula
@@ -273,28 +311,39 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
   names(nc) <- names(meanind) <- distnames
   for(i in distnames){
     nc[[i]] <- apply(fullDM[[i]],1:2,function(x) !all(unlist(x)==0))
-    if(m$conditions$circularAngleMean[[i]]) meanind[[i]] <- which((apply(fullDM[[i]][1:nbStates,,drop=FALSE],1,function(x) !all(unlist(x)==0))))
+    if(m$conditions$circularAngleMean[[i]]) {
+      meanind[[i]] <- which((apply(fullDM[[i]][1:nbStates,,drop=FALSE],1,function(x) !all(unlist(x)==0))))
+      # deal with angular covariates that are exactly zero
+      if(length(meanind[[i]])){
+        angInd <- which(is.na(match(gsub("cos","",gsub("sin","",colnames(nc[[i]]))),colnames(nc[[i]]),nomatch=NA)))
+        sinInd <- colnames(nc[[i]])[which(grepl("sin",colnames(nc[[i]])[angInd]))]
+        nc[[i]][meanind[[i]],sinInd]<-ifelse(nc[[i]][meanind[[i]],sinInd],nc[[i]][meanind[[i]],sinInd],nc[[i]][meanind[[i]],gsub("sin","cos",sinInd)])
+        nc[[i]][meanind[[i]],gsub("sin","cos",sinInd)]<-ifelse(nc[[i]][meanind[[i]],gsub("sin","cos",sinInd)],nc[[i]][meanind[[i]],gsub("sin","cos",sinInd)],nc[[i]][meanind[[i]],sinInd])
+      }
+    }
   }
   
   Par$real<-list()
   for(i in distnames){
+    tmpParNames <- p$parNames[[i]]
+    tmpParNames[which(p$parNames[[i]]=="kappa")] <- "concentration"
     
     DMind[[i]] <- FALSE
-    par <- c(w2n(miBeta$coefficients,p$bounds,p$parSize,nbStates,nbCovs,m$conditions$estAngleMean,m$conditions$circularAngleMean,m$conditions$stationary,DMinputs$cons,fullDM,DMind,DMinputs$workcons,1,dist[i],m$conditions$Bndind,nc,meanind,m$covsDelta)[[i]])
+    par <- c(w2n(miBeta$coefficients,p$bounds,p$parSize,nbStates,nbCovs,m$conditions$estAngleMean,m$conditions$circularAngleMean[i],inputs$consensus[i],m$conditions$stationary,DMinputs$cons,fullDM,DMind,DMinputs$workcons,1,inputs$dist[i],m$conditions$Bndind,nc,meanind,m$covsDelta,NULL)[[i]])
 
-    if(!(dist[[i]] %in% angledists) | (dist[[i]] %in% angledists & m$conditions$estAngleMean[[i]] & !m$conditions$Bndind[[i]])) {
-      Par$real[[i]] <- get_CI(miBeta$coefficients,par,m,parindex[[i]]+1:ncol(fullDM[[i]]),fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],nbStates,alpha,p$parNames[[i]],m$stateNames,nc[[i]],meanind[[i]])
+    if(!(inputs$dist[[i]] %in% angledists) | (inputs$dist[[i]] %in% angledists & m$conditions$estAngleMean[[i]] & !m$conditions$Bndind[[i]])) {
+      Par$real[[i]] <- get_CI(miBeta$coefficients,par,m,parindex[[i]]+1:parCount[[i]],fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],inputs$consensus[[i]],nbStates,alpha,tmpParNames,m$stateNames,nc[[i]],meanind[[i]],NULL)
     } else {
       if(!m$conditions$estAngleMean[[i]]){
-        Par$real[[i]] <- get_CI(miBeta$coefficients,par[-(1:nbStates)],m,parindex[[i]]+1:ncol(fullDM[[i]]),fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],nbStates,alpha,p$parNames[[i]],m$stateNames,nc[[i]],meanind[[i]])
+        Par$real[[i]] <- get_CI(miBeta$coefficients,par[-(1:nbStates)],m,parindex[[i]]+1:parCount[[i]],fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],inputs$consensus[[i]],nbStates,alpha,tmpParNames,m$stateNames,nc[[i]],meanind[[i]],NULL)
         Par$real[[i]]$est <- matrix(c(rep(0,nbStates),Par$real[[i]]$est),ncol=nbStates,byrow=T)
         Par$real[[i]]$se <- matrix(c(rep(NA,nbStates),Par$real[[i]]$se),ncol=nbStates,byrow=T)
         Par$real[[i]]$lower <- matrix(c(rep(NA,nbStates),Par$real[[i]]$lower),ncol=nbStates,byrow=T)
         Par$real[[i]]$upper <- matrix(c(rep(NA,nbStates),Par$real[[i]]$upper),ncol=nbStates,byrow=T)  
-        dimnames(Par$real[[i]]$est) <- dimnames(Par$real[[i]]$se) <- dimnames(Par$real[[i]]$lower) <- dimnames(Par$real[[i]]$upper) <- list(c("mean",p$parNames[[i]]),m$stateNames)
+        dimnames(Par$real[[i]]$est) <- dimnames(Par$real[[i]]$se) <- dimnames(Par$real[[i]]$lower) <- dimnames(Par$real[[i]]$upper) <- list(c("mean",tmpParNames),m$stateNames)
       } else {
         if(m$conditions$Bndind[[i]]){
-          Par$real[[i]] <- CI_angle(miBeta$coefficients,par,m,parindex[[i]]+1:ncol(fullDM[[i]]),fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],nbStates,alpha,p$parNames[[i]],m$stateNames,nc[[i]],meanind[[i]])
+          Par$real[[i]] <- CI_angle(miBeta$coefficients,par,m,parindex[[i]]+1:parCount[[i]],fullDM[[i]],DMind[[i]],p$bounds[[i]],DMinputs$cons[[i]],DMinputs$workcons[[i]],miBeta$variance,m$conditions$circularAngleMean[[i]],inputs$consensus[[i]],nbStates,alpha,tmpParNames,m$stateNames,nc[[i]],meanind[[i]],NULL)
         }
       }
     }
@@ -442,7 +491,10 @@ MIpool<-function(HMMfits,alpha=0.95,ncores,covs=NULL){
       cat("Calculating location",paste0(alpha*100,"%"),"error ellipses... ")
       registerDoParallel(cores=ncores)
       errorEllipse<-foreach(i = 1:nrow(mh$data)) %dopar% {
-        car::dataEllipse(cbind(unlist(lapply(im,function(x) x$data$x[i])),unlist(lapply(im,function(x) x$data$y[i]))),levels=alpha,draw=FALSE,segments=100)
+        tmp <- cbind(unlist(lapply(im,function(x) x$data$x[i])),unlist(lapply(im,function(x) x$data$y[i])))
+        if(length(unique(tmp[,1]))>1 | length(unique(tmp[,2]))>1)
+          ellip <- car::dataEllipse(tmp,levels=alpha,draw=FALSE,segments=100)
+        else ellip <- matrix(tmp[1,],101,2,byrow=TRUE)
       }
       stopImplicitCluster()
       cat("DONE\n")
