@@ -30,12 +30,24 @@ checkInputs<-function(nbStates,dist,Par,estAngleMean,circularAngleMean,zeroInfla
   } else {
     if(!is.list(circularAngleMean) | is.null(names(circularAngleMean))) stop("'circularAngleMean' must be a named list")
   }
+
+  consensus <- vector('list',length(distnames))
+  names(consensus) <- distnames
+  
+  ndist <- dist
   for(i in distnames){
+    if(dist[[i]]=="vmConsensus"){
+      consensus[[i]] <- TRUE
+      estAngleMean[[i]] <- circularAngleMean[[i]] <- TRUE
+      if(is.null(DM[[i]])) stop("DM$",i," must be specified when dist$",i,"=vmConsensus")
+      ndist[[i]] <- gsub("Consensus","",dist[[i]])
+    } else consensus[[i]] <- FALSE
     if(is.null(circularAngleMean[[i]]) | !estAngleMean[[i]]) circularAngleMean[[i]] <- FALSE
     if(!is.logical(circularAngleMean[[i]])) stop("circularAngleMean$",i," must be logical")
     if(circularAngleMean[[i]] & is.null(DM[[i]])) stop("DM$",i," must be specified when circularAngleMean$",i,"=TRUE")
   }
   circularAngleMean<-circularAngleMean[distnames]
+  consensus<-consensus[distnames]
   
   if(!is.null(stateNames) & length(stateNames)!=nbStates)
     stop("stateNames must have length ",nbStates)
@@ -71,5 +83,5 @@ checkInputs<-function(nbStates,dist,Par,estAngleMean,circularAngleMean,zeroInfla
     #if(!is.null(DM[[i]]) !is.null(userBounds[[i]])) stop("either userBounds$",i," or DM$",i," must be NULL")
   }
 
-  return(list(p=p,estAngleMean=estAngleMean,circularAngleMean=circularAngleMean,DM=DM,cons=cons,workcons=workcons))
+  return(list(p=p,dist=ndist,estAngleMean=estAngleMean,circularAngleMean=circularAngleMean,consensus=consensus,DM=DM,cons=cons,workcons=workcons))
 }
\ No newline at end of file
